package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.marking.EvaluationParameters;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.ExamPaperStructure;
import com.qmth.distributed.print.business.service.DataSyncService;
import com.qmth.distributed.print.business.service.ExamPaperStructureService;
import com.qmth.teachcloud.common.bean.params.BasicMajorParams;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 试卷结构 前端控制器
 * </p>
 *
 * @author xf
 */
@Api(tags = "试卷结构Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.exam}/structure")
public class ExamPaperStructureController {

    @Resource
    private ExamPaperStructureService examPaperStructureService;

    @Resource
    private DataSyncService dataSyncService;

    /**
     * 查询
     *
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@ApiParam(value = "学期ID", required = false) @RequestParam(required = false) Long semesterId,
                       @ApiParam(value = "考试ID", required = false) @RequestParam(required = false) Long examId,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<ExamPaperStructure> examPaperStructureIPage = examPaperStructureService.listByPropositionTeacherId(semesterId, examId, pageNumber, pageSize, null);
        return ResultUtil.ok(examPaperStructureIPage);
    }


    /**
     * 上传
     *
     * @param md5   文件md5
     * @param files 文件数组
     * @return Result
     */
    @ApiOperation(value = "上传试卷结构、标答")
    @RequestMapping(value = "/upload", method = RequestMethod.POST)
    public Result upload(@RequestParam("examPaperStructure") String examPaperStructure,
                         @RequestParam("md5") String md5,
                         @RequestParam("keys") String keys,
                         @RequestParam("files") MultipartFile[] files) {
        ExamPaperStructure examPaper = examPaperStructureService.upload(examPaperStructure, md5, keys, files);
        // 推送云阅卷
        dataSyncService.syncPaperStructure(examPaper);
        return ResultUtil.ok(String.valueOf(examPaper.getId()), null);
    }

    @ApiOperation(value = "评卷参数-提交")
    @RequestMapping(value = "/submit", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = EditResult.class)})
    public Result submitExamPaperParams(@Valid @RequestBody EvaluationParameters evaluationParameters, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        ExamPaperStructure examPaperStructure = examPaperStructureService.submitExamPaperParams(evaluationParameters,sysUser);
        // 异步推送云阅卷
        return ResultUtil.ok();
    }

    /**
     * 上传标答文件
     *
     * @param md5   文件md5
     * @param file 文件数组
     * @return Result
     */
    @ApiOperation(value = "上传试卷标答文件")
    @RequestMapping(value = "/upload_answer", method = RequestMethod.POST)
    public Result uploadAnswer(@RequestParam("examPaperStructure") String examPaperStructure,
                         @RequestParam("md5") String md5,
                         @RequestParam("file") MultipartFile file) {
        ExamPaperStructure examPaper = examPaperStructureService.uploadAnswer(examPaperStructure, md5, file);
        // 推送云阅卷
        dataSyncService.syncPaperStructure(examPaper);
        return ResultUtil.ok(String.valueOf(examPaper.getId()), null);
    }

    /**
     * 试卷结构预览
     *
     * @param id id
     */
    @ApiOperation(value = "试卷结构预览")
    @RequestMapping(value = "/preview_structure", method = RequestMethod.POST)
    public Result preStructure(@RequestParam("id") Long id) {
        List<Map> list = examPaperStructureService.preStructure(id);
        return ResultUtil.ok(list);
    }



}

